// Generated by CoffeeScript 1.7.1
(function() {
  var Bottleneck,
    __slice = [].slice;

  Bottleneck = (function() {
    Bottleneck.strategy = Bottleneck.prototype.strategy = {
      LEAK: 1,
      OVERFLOW: 2,
      BLOCK: 3
    };

    function Bottleneck(maxNb, minTime, highWater, strategy) {
      this.maxNb = maxNb != null ? maxNb : 0;
      this.minTime = minTime != null ? minTime : 0;
      this.highWater = highWater != null ? highWater : 0;
      this.strategy = strategy != null ? strategy : Bottleneck.prototype.strategy.LEAK;
      this._nextRequest = Date.now();
      this._nbRunning = 0;
      this._queue = [];
      this._timeouts = [];
      this._unblockTime = 0;
      this.penalty = (15 * this.minTime) || 5000;
      this.interrupt = false;
      this.reservoir = null;
    }

    Bottleneck.prototype.check = function() {
      return (this._nbRunning < this.maxNb || this.maxNb <= 0) && (this._nextRequest - Date.now()) <= 0 && ((this.reservoir == null) || this.reservoir > 0);
    };

    Bottleneck.prototype._tryToRun = function() {
      var done, index, next, wait;
      if ((this._nbRunning < this.maxNb || this.maxNb <= 0) && this._queue.length > 0 && ((this.reservoir == null) || this.reservoir > 0)) {
        this._nbRunning++;
        if (this.reservoir != null) {
          this.reservoir--;
        }
        wait = Math.max(this._nextRequest - Date.now(), 0);
        this._nextRequest = Date.now() + wait + this.minTime;
        next = this._queue.shift();
        done = false;
        index = -1 + this._timeouts.push(setTimeout((function(_this) {
          return function() {
            return next.task.apply({}, next.args.concat(function() {
              var _ref;
              if (!done) {
                done = true;
                delete _this._timeouts[index];
                _this._nbRunning--;
                _this._tryToRun();
                if (!_this.interrupt) {
                  return (_ref = next.cb) != null ? _ref.apply({}, Array.prototype.slice.call(arguments, 0)) : void 0;
                }
              }
            }));
          };
        })(this), wait));
        return true;
      } else {
        return false;
      }
    };

    Bottleneck.prototype.submit = function() {
      var args, cb, reachedHighWaterMark, task, _i;
      task = arguments[0], args = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), cb = arguments[_i++];
      reachedHighWaterMark = this.highWater > 0 && this._queue.length === this.highWater;
      if (this.strategy === Bottleneck.prototype.strategy.BLOCK && (reachedHighWaterMark || this._unblockTime >= Date.now())) {
        this._unblockTime = Date.now() + this.penalty;
        this._nextRequest = this._unblockTime + this.minTime;
        return true;
      } else if (reachedHighWaterMark) {
        if (this.strategy === Bottleneck.prototype.strategy.LEAK) {
          this._queue.shift();
        } else if (this.strategy === Bottleneck.prototype.strategy.OVERFLOW) {
          return reachedHighWaterMark;
        }
      }
      this._queue.push({
        task: task,
        args: args,
        cb: cb
      });
      this._tryToRun();
      return reachedHighWaterMark;
    };

    Bottleneck.prototype.changeSettings = function(maxNb, minTime, highWater, strategy) {
      this.maxNb = maxNb != null ? maxNb : this.maxNb;
      this.minTime = minTime != null ? minTime : this.minTime;
      this.highWater = highWater != null ? highWater : this.highWater;
      this.strategy = strategy != null ? strategy : this.strategy;
      while (this._tryToRun()) {}
      return this;
    };

    Bottleneck.prototype.changePenalty = function(penalty) {
      this.penalty = penalty != null ? penalty : this.penalty;
      return this;
    };

    Bottleneck.prototype.changeReservoir = function(reservoir) {
      this.reservoir = reservoir;
      while (this._tryToRun()) {}
      return this;
    };

    Bottleneck.prototype.incrementReservoir = function(incr) {
      if (incr == null) {
        incr = 0;
      }
      this.changeReservoir(this.reservoir + incr);
      return this;
    };

    Bottleneck.prototype.stopAll = function(interrupt) {
      var a, _i, _len, _ref;
      this.interrupt = interrupt != null ? interrupt : this.interrupt;
      _ref = this._timeouts;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        a = _ref[_i];
        clearTimeout(a);
      }
      this._tryToRun = function() {};
      this.submit = function() {
        return false;
      };
      return this.check = function() {
        return false;
      };
    };

    return Bottleneck;

  })();

  module.exports = Bottleneck;

}).call(this);
